wf    = require "windfield"
candy = require "candy"

function love.load()
    gr = love.graphics
    kb = love.keyboard
    mo = love.mouse
    to = love.touch
    ma = love.math

    colors = {
        black = {0, 0, 0},
        darkGray = {48, 48, 48},
        white = {255, 255, 255},
        almostWhite = {248, 248, 248},
        red = {208, 16, 0},
        green = {0, 104, 0},
        lightGreen = {40, 192, 32},
    }

    win = {w = gr.getWidth(), h = gr.getHeight()}
    canvasWidth = 144
    canvasHeight = 256
    canvas = gr.newCanvas(canvasWidth, canvasHeight, "normal")
    sx = win.w / canvasWidth -- x scale
    sy = win.h / canvasHeight -- y scale

    wt = 60 -- wall thickness
    cr = 13 -- candy radius

    world = wf.newWorld(0, 0, true)
    world:setGravity(0, 512)

    --ball = world:newCircleCollider(canvasWidth / 2, canvasHeight / 2, 10)
    --ball:setRestitution(0.4)
    --ball:applyAngularImpulse(5000)
    candies = {}

    wallTop    = world:newRectangleCollider(0, -wt, canvasWidth, wt)
    wallBottom = world:newRectangleCollider(0, canvasHeight, canvasWidth, wt)
    wallLeft   = world:newRectangleCollider(-wt, -wt, wt, canvasHeight + wt * 2)
    wallRight   = world:newRectangleCollider(canvasWidth, -wt, wt, canvasHeight + wt * 2)
    
    wallTop:setType("static")
    wallBottom:setType("static")
    wallLeft:setType("static")
    wallRight:setType("static")



    MOUSE_LEFT = 1
    MOUSE_RIGHT = 2
    MOUSE_MIDDLE = 3



    timerEnterCandy = 0
    delayEnterCandy = .2
end

function love.update(dt)
    world:update(dt)

    timerEnterCandy = timerEnterCandy + dt

    if timerEnterCandy >= delayEnterCandy then
        timerEnterCandy = 0

        local newCandy = candy.new(
            ma.random(cr, canvasWidth - cr),
            ma.random(cr, 50),
            cr
        )

        newCandy.body = world:newCircleCollider(
            newCandy.x,
            newCandy.y,
            newCandy.radius
        )

        table.insert(candies, newCandy)
    end

    -- update candies:
    for k, candy in ipairs(candies) do
        candy.x, candy.y = candy.body:getPosition()
    end
end

function love.draw()
    gr.setColor(colors.white)
    gr.setCanvas(canvas)

    gr.setColor(colors.black)
    gr.rectangle("fill", 0, 0, canvasWidth, canvasHeight)
    --debug:
    --world:draw()

    -- draw candies:
    for k, candy in ipairs(candies) do
        gr.setColor(colors.red)
        if candy.selected then
            gr.setColor(colors.lightGreen)
        end
        gr.circle("line", candy.x, candy.y, candy.radius)

        if mo.isDown(MOUSE_LEFT) then
            local selectedCandy = getSelectedCandy()

            if selectedCandy then
                gr.line(
                    selectedCandy.x,
                    selectedCandy.y,
                    mo.getX() / sx,
                    mo.getY() / sy)
            end
        end
    end

    gr.setCanvas()
    gr.setColor(colors.white)
    gr.draw(canvas, 0, 0, 0, sx, sy)

    gr.print(fps())
end

function love.keypressed(k)
    if k == "escape" then
        quit()
    end
end

function love.mousepressed(x, y, btn)
    if btn == MOUSE_LEFT then
        for k, candy in ipairs(candies) do
            if candy:overIt(mo.getX() / sx, mo.getY() / sy) then
                candy.selected = true
            else
                candy.selected = false
            end
        end
    end
end

function love.mousereleased(x, y, btn)
    if btn == MOUSE_LEFT then
        for k, candy in ipairs(candies) do
            local selectedCandy = getSelectedCandy()

            if selectedCandy then
                selectedCandy.body:applyLinearImpulse(
                    -(selectedCandy.x - mo.getX() / sx) / 2,
                    -(selectedCandy.y - mo.getY() / sy) / 2
                )
            end
        end
    end
end








function getSelectedCandy()
    for k, candy in ipairs(candies) do
        if candy.selected then
            return candy
        end
    end
end

function fps()
    return love.timer.getFPS()
end

function quit()
    love.event.quit()
end
